if not HolySonar then return end
local HolySonar = HolySonar

local AceDB = LibStub("AceDB-3.0") -- Makes saving things really easy

local UnitHealth, UnitHealthMax, UnitExists, UnitIsDeadOrGhost, UnitIsUnit, GetPlayerFacing, IsInRaid, GetNumGroupMembers, GetSpellBonusHealing, GetMasteryEffect, UnitPosition, UnitPositionSquared =
		UnitHealth, UnitHealthMax, UnitExists, UnitIsDeadOrGhost, UnitIsUnit, GetPlayerFacing, IsInRaid, GetNumGroupMembers, GetSpellBonusHealing, GetMasteryEffect, UnitPosition, UnitPositionSquared
local tostring, tonumber, print, abs, sin, cos, math, type, pairs = tostring, tonumber, print, math.abs, math.sin, math.cos, math, type, pairs

local TwistofFate = GetSpellInfo(123254)
local Archangel = GetSpellInfo(81700)


-- Constants
local HolyNovaRadius = 12
local MapErrorFactor = 0.95 -- should be below 1 to be sure the shown number of targets will be healed

local pi = math.pi

-- This Global will keep all dots frames, so change them to update the radar 
local dots = {} 


---------------------------
---- Utility functions ----
--------------------------

function HolySonar:getOffset(region)
 local _, _, anchor, x, y = region:GetPoint()
 return tonumber(x), tonumber(y)
end


local ninetyDegreeAngleInRadians = (pi / 2) 
function HolySonar:getDistance(x1, y1, x2, y2)
	local xx = x2 - x1
	local yy = y2 - y1
	return sqrt(xx*xx + yy*yy)
end


-- Unit HP Function, all in One
local function UnitHP(unit)
	local maxHp = UnitHealthMax(unit)
	local currentHp = UnitHealth(unit)

	if maxHp == 0 then
		maxHp = 1
	end
	if currentHp == 0 then
		currentHp = 1
	end
 
	local pctHp = currentHp/maxHp
 
	return pctHp, maxHp-currentHp
end

-- Deficit color
-- Maybe add the possibility for a player to create various ranges?
-- No need, I think. Changing colors should be enoguh, maybe only percentages fix
local function getDeficitColor(HP) 
	if(HP >= 0.95) then 
		return HolySonar.db.profile.radar.color.green 
	elseif(HP < 0.95 and HP >= 0.75) then 
		return HolySonar.db.profile.radar.color.yellow 
	elseif(HP < 0.75 and HP >= 0.50) then 
		return HolySonar.db.profile.radar.color.orange 
	elseif(HP < 0.50) then 
		return HolySonar.db.profile.radar.color.red 
	end 
end 


function HolySonar:getHolyNovaHealingValue() 
	local mastery = GetMasteryEffect()
	local SP = GetSpellBonusHealing()
	local Versatility = 1 + GetCombatRatingBonus(CR_VERSATILITY_DAMAGE_DONE)/100
	local Grace = 1.3
	local HNcoefficient = 1.125 
	local AA = 1.0
	local ToF = 1.0

	
	if UnitBuff("player", TwistofFate) then
		ToF = 1.15 
	end

	local modifier = select(15,UnitBuff("player", Archangel))
	if modifier then 
		AA = (1 + modifier / 100) 	
	end
	HNHeal = ((SP * HNcoefficient)*(mastery/200+1)*Versatility*Grace*ToF*AA)
	return HNHeal
end


-- Holy Nova Icon With text initialization!
local function newHNIcon()
	-- Create the main frame
	local IconFrame = CreateFrame("Frame", "HolyNovaTargets", UIParent)
	IconFrame:SetFrameStrata("MEDIUM")
	IconFrame:SetClampedToScreen(true)
	IconFrame:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.icon.offsetX, HolySonar.db.profile.icon.offsetY);
	IconFrame:SetSize(HolySonar.db.profile.icon.size, HolySonar.db.profile.icon.size)
	IconFrame:EnableMouse(true)
	IconFrame:SetMovable(true)
	IconFrame:RegisterForDrag("LeftButton")
	IconFrame:SetScript("OnDragStart", function()
		if not HolySonar.db.profile.lock then
			IconFrame:StartMoving()
		end
	end)
	IconFrame:SetScript("OnDragStop",   
		function() -- dragging stopped, save the new position values
			IconFrame:StopMovingOrSizing()
			HolySonar.db.profile.icon.offsetX, HolySonar.db.profile.icon.offsetY = HolySonar:getOffset(IconFrame)                 
			HolySonar:optionsChanged()
		end)
	
	
	-- Set Holy Nova Icon Texture
	local iconTexture = IconFrame:CreateTexture(nil,"BACKGROUND")
	iconTexture:SetAllPoints(IconFrame) 
	iconTexture:SetTexture("Interface\\Icons\\Spell_Holy_HolyNova.blp") 

	-- Overlay on the icon to display a "Good/Bad" effect
	local overlay = IconFrame:CreateTexture("ARTWORK") 
	overlay:SetAllPoints()
	-- Start with BAD Color
	overlay:SetTexture(HolySonar.db.profile.icon.color.BAD.r, HolySonar.db.profile.icon.color.BAD.g, HolySonar.db.profile.icon.color.BAD.b, HolySonar.db.profile.icon.color.BAD.a)
	overlay:Show()
	IconFrame.overlay = overlay

	-- Text to display amount if people around
	local iconText = IconFrame:CreateFontString();
	iconText:SetFont("Fonts\\FRIZQT__.TTF", HolySonar.db.profile.icon.fontSize, "OUTLINE, MONOCHROME")
	iconText:SetText("0")
	iconText:SetTextColor(1,1,1)
	iconText:SetPoint("CENTER",IconFrame,"CENTER",0,0)
	IconFrame.text = iconText
	IconFrame:Show()

	function IconFrame:Resize(size)
		self:SetWidth(size); 
		self:SetHeight(size);
		self.overlay:SetWidth(size)
		self.overlay:SetHeight(size)
	end

	function IconFrame:ApplyOptions() -- applies the current options to the frame (size, scale, offset)
		self:Resize(HolySonar.db.profile.icon.size)
		self:SetScale(HolySonar.db.profile.icon.scale)
		self:ClearAllPoints()
		self:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.icon.offsetX, HolySonar.db.profile.icon.offsetY);
		self.text:SetFont("Fonts\\FRIZQT__.TTF", HolySonar.db.profile.icon.fontSize, "OUTLINE, MONOCHROME")
		if HolySonar.db.profile.icon.enable then
			self:Show()
		else
			self:Hide()
		end 
		if HolySonar.db.profile.lock then
			IconFrame:EnableMouse(false)
		else
			IconFrame:EnableMouse(true)
		end 

	end 
	
	return IconFrame
end

-- Radar Frame initialization code!
local function newRadar() 

	local radarFrame = CreateFrame("Frame", "LifeRadar", UIParent) 
	radarFrame:SetFrameStrata("MEDIUM")
	radarFrame:SetClampedToScreen(true)
	radarFrame:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.radar.offsetX, HolySonar.db.profile.radar.offsetY);
	radarFrame:SetSize(HolySonar.db.profile.radar.size,HolySonar.db.profile.radar.size) 
	radarFrame:EnableMouse(true) -- not click through
	radarFrame:SetMovable(true)
	radarFrame:RegisterForDrag("LeftButton")
	radarFrame:SetScript("OnDragStart", function()
		if not HolySonar.db.profile.lock then
			radarFrame:StartMoving()
		end
	end)
	radarFrame:SetScript("OnDragStop", 
		function() -- dragging stopped, save the new position values
			radarFrame:StopMovingOrSizing()
			HolySonar.db.profile.radar.offsetX, HolySonar.db.profile.radar.offsetY = HolySonar:getOffset(radarFrame)     
			HolySonar:optionsChanged()
		end)
	

	-- Transparent Black background 
	local bg = radarFrame:CreateTexture(nil, "BACKGROUND") 
	bg:SetAllPoints(radarFrame) 
	bg:SetBlendMode("BLEND") 
	bg:SetTexture(HolySonar.db.profile.radar.color.background.r, HolySonar.db.profile.radar.color.background.g, HolySonar.db.profile.radar.color.background.b,HolySonar.db.profile.radar.color.background.a) 
	radarFrame.background = bg 


	-- You can create another one and import it 
	local circle = radarFrame:CreateTexture(nil, "ARTWORK") 
	circle:SetSize(HolySonar.db.profile.radar.size, HolySonar.db.profile.radar.size) 
	circle:SetPoint("CENTER") 
	circle:SetTexture(HolySonar.db.profile.radar.circleTexture) 
	circle:SetBlendMode("ADD") 
	circle:SetVertexColor(HolySonar.db.profile.radar.color.radar.r, HolySonar.db.profile.radar.color.radar.g, HolySonar.db.profile.radar.color.radar.b,HolySonar.db.profile.radar.color.radar.a) 
	-- Store the background circle on the variable, so we can change it's color
	radarFrame.circle = circle 

	-- Line overlay on the radar!
	local lines = radarFrame:CreateTexture(nil, "ARTWORK") 
	lines:SetSize(HolySonar.db.profile.radar.size, HolySonar.db.profile.radar.size) 
	lines:SetAllPoints(circle) 
	lines:SetTexture(HolySonar.db.profile.radar.linesTexture) 
	lines:SetBlendMode("ADD") 
	lines:SetVertexColor(HolySonar.db.profile.radar.color.lines.r, HolySonar.db.profile.radar.color.lines.g, HolySonar.db.profile.radar.color.lines.b,HolySonar.db.profile.radar.color.lines.a)
	-- Store the lines on the variable, so we can change it later
	radarFrame.lines = lines 

	-- Player arrow, not needed, but looks good for reference
	local player = radarFrame:CreateTexture(nil, "OVERLAY") 
	player:SetSize(32, 32) 
	player:SetTexture("Interface\\Minimap\\MinimapArrow.blp") 
	player:SetBlendMode("ADD") 
	player:SetPoint("CENTER") 
	radarFrame.player = player

	-- Max Raid size is 40, so 40 frames (Dots) created should be enough
	for i = 1, 40 do 
		local dot = radarFrame:CreateTexture(nil, "OVERLAY") 
		dot:SetSize(20*HolySonar.db.profile.radar.dotScale, 20*HolySonar.db.profile.radar.dotScale) 
		dot:SetTexture("Interface\\Minimap\\PartyRaidBlips") 
		dot:SetTexCoord(0.5, 0.625, 0, 0.25) 

		-- Here you can set the dot color using RGBA, so the icon original color is irrelevant 
		-- It'll desaturate it and colorize 
		-- Randomized for fun/example
		--dot:SetVertexColor(math.random(),math.random(),math.random(),1) 

		dot:Hide() 
		dots[i] = dot 
	end 

	function radarFrame:Resize(size)
		self:SetWidth(size); 
		self:SetHeight(size);       
		self.background:SetWidth(size); 
		self.background:SetHeight(size);
		self.circle:SetWidth(size); 
		self.circle:SetHeight(size);
	end

	function radarFrame:ApplyOptions() -- applies the current options to the frame (size, scale, offset)
		--self:Resize(HolySonar.db.profile.radar.size)
		self:SetScale(HolySonar.db.profile.radar.scale)
		self:ClearAllPoints()
		self:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.radar.offsetX, HolySonar.db.profile.radar.offsetY);
		if HolySonar.db.profile.radar.enable then
			self:Show()
		else
			self:Hide()
		end 
		if HolySonar.db.profile.lock then
			radarFrame:EnableMouse(false)
		else
			radarFrame:EnableMouse(true)
		end 
		
		-- colors
		self.circle:SetVertexColor(HolySonar.db.profile.radar.color.radar.r, HolySonar.db.profile.radar.color.radar.g, HolySonar.db.profile.radar.color.radar.b, HolySonar.db.profile.radar.color.radar.a) 
		self.background:SetTexture(HolySonar.db.profile.radar.color.background.r, HolySonar.db.profile.radar.color.background.g, HolySonar.db.profile.radar.color.background.b,HolySonar.db.profile.radar.color.background.a)
		self.lines:SetVertexColor(HolySonar.db.profile.radar.color.lines.r, HolySonar.db.profile.radar.color.lines.g, HolySonar.db.profile.radar.color.lines.b, HolySonar.db.profile.radar.color.lines.a) 
		
		-- textures
		circle:SetTexture(HolySonar.db.profile.radar.circleTexture)
		lines:SetTexture(HolySonar.db.profile.radar.linesTexture) 
	end 
		
	-- Frame starts hidden
	radarFrame:Show() 

	local total_Data = 0
	function radarFrame:onUpdate(elapsed)
		total_Data = total_Data + elapsed
		if total_Data >= 0.05 then -- The pitch always needs to update if enabled
			if HolySonar.db.profile.radar.pitch then
				local y_compression = sin(math.max(-GetUnitPitch("player"), .4)) or 1
				local size = HolySonar.db.profile.radar.size
				local height = size * y_compression
				HolySonar.Radar.circle:SetSize(size, height)
			else
				local size = HolySonar.db.profile.radar.size
				HolySonar.Radar.circle:SetSize(size, size)
			end
		end
		if total_Data >= HolySonar.db.profile.refreshRate  then
			if HolySonar.db.profile.radar.enable or HolySonar.db.profile.icon.enable then
				HolySonar:UpdatePinMap()    
			end
			
			total_Data = 0
		end
	end
	return radarFrame 
end

function HolySonar:UpdatePinMap()

	-- Holy Nova Heal amount
	local HolyNovaAmount = self:getHolyNovaHealingValue()

	local playersInRange = 0
	local playerX, playerY = UnitPosition("player")
	local numMembers = GetNumGroupMembers()

	local prefix = "raid"
	-- If the player is in a party, he's not on the Unit ID list
	if not IsInRaid() then
		prefix = "party"
		numMembers = numMembers - 1
	end

	local last = 1
	local rotation = (1.5 * pi) - GetPlayerFacing() --*57.2957795
	for i=1, numMembers do
		-- String.Format is faster than concatenating strings - Or is it?
		--local unit = prefix..i
		local unit = format("%s%d", prefix, i)

		-- Don't consider units that are Dead or disconnected
		-- Also, don't include the player here, he'll be taken in account after this for iteration
		if UnitExists(unit) and not UnitIsDeadOrGhost(unit) and not UnitIsUnit(unit,"player") then
			local pctHp, difHp = UnitHP(unit)
			unitX, unitY = UnitPosition(unit)

			-- All in Yards! (Hopefully)
			--local distance = HolySonar:getDistance(playerX,playerY,unitX,unitY)
			local distance = math.sqrt(UnitDistanceSquared(unit))
			-- Check if the HP Deficit is bigger than what we're looking to heal!
			if difHp > HolyNovaAmount*HolySonar.db.profile.HolyNovaOverhealCoheficient then
				if distance <= HolyNovaRadius*MapErrorFactor then
					 playersInRange = playersInRange + 1
				end
			end
			
			 -- Get a dot
			local dot = dots[i]
		   
			-- Get dot color based on player Health percent
			local color = getDeficitColor(pctHp)
			dot:SetVertexColor(color.r,color.g,color.b,1)

			-- Calculate Radar Coordinates
			local dist_x = playerX - unitX
			local dist_y = playerY - unitY
	 
			-- Normalize it to pixels
			-- How much does 1 yard is in pixels for our radar size? RadarRadius/Measuring radius
			local yardInPixel = (HolySonar.db.profile.radar.size/(HolyNovaRadius*2))
	 
			-- Multiply it so we get the values in pixel
			-- Also factor in the player rotation!

			local radar_x, radar_y, y_compression = 0,0,1
			if HolySonar.db.profile.radar.pitch then
				y_compression = sin(math.max(-GetUnitPitch("player"), .4)) or 1
			end
			if HolySonar.db.profile.radar.rotate then
				radar_x = ((dist_x * cos(rotation)) - (dist_y * sin(rotation))) * yardInPixel
				radar_y = ((dist_x * sin(rotation)) + (dist_y * cos(rotation))) * yardInPixel * y_compression
			else
				radar_x = dist_x * yardInPixel
				radar_y = dist_y * yardInPixel * y_compression
			end

			--print(unit..": ( "..unitX..","..unitY..") ("..radar_x..","..radar_y..")")
			-- Since we considered the player is in the center, we set the anchor to the center of the radar
			-- and place our offsets for x and y as calculated
			-- Need to clear point always
			dot:ClearAllPoints();
			dot:SetPoint("CENTER", self.Radar, "CENTER", radar_x, radar_y)
			
			if HolySonar.db.profile.radar.showOutsideRadar then
				if distance <= HolyNovaRadius + HolySonar.db.profile.radar.outsideRange then
					dot:Show()
				else
					dot:Hide()
				end
			else
				if distance <= HolyNovaRadius then
					dot:Show()
				else
					dot:Hide()
				end
			end
		else
			dots[i]:Hide()
		end

		-- If this is the last, then everything beyond should be hidden (see below)
		last = i + 1
	end
	
	-- Account the player on the "players in range" count
	local _,difHp = UnitHP("player")
	if difHp > HolyNovaAmount*HolySonar.db.profile.HolyNovaOverhealCoheficient then
		playersInRange = playersInRange + 1
	end

	-- Hide all unused dots
	for i=last, 40 do 
		dots[i]:Hide() 
	end

	-- Set the amount on the icon   
	self.HolyNovaIcon.text:SetText(playersInRange)
	
	-- Set the overlay based on the number of players in range
	local overlay = self.HolyNovaIcon.overlay
	if playersInRange > 4 then
		overlay:SetTexture(HolySonar.db.profile.icon.color.GOOD.r, HolySonar.db.profile.icon.color.GOOD.g, HolySonar.db.profile.icon.color.GOOD.b, HolySonar.db.profile.icon.color.GOOD.a)
	else
		overlay:SetTexture(HolySonar.db.profile.icon.color.BAD.r, HolySonar.db.profile.icon.color.BAD.g, HolySonar.db.profile.icon.color.BAD.b, HolySonar.db.profile.icon.color.BAD.a)
	end
end

------------------
--- Init Addon ---
------------------

function HolySonar:OnProfileChanged(val1, val2, val3)
	-- TODO
end

function HolySonar:EventCapture(this, event, ...)
	local unit = ...
	if event == "PLAYER_LOGIN" or event == "PLAYER_SPECIALIZATION_CHANGED" and unit == "player" then
		if select(2,UnitClass("player")) ~= "PRIEST" or GetSpecialization() ~= 1 then
			self.frame:Hide() -- Hiding OnUpdate frame, stops event calls.
			self.Radar:Hide()
			self.HolyNovaIcon:Hide()
		else
			self.frame:Show()
			self.Radar:ApplyOptions()
			self.HolyNovaIcon:ApplyOptions()
		end
	end 
end

function HolySonar:OnInitialize()
	self:RegisterChatCommand("hns", "SlashProcessor_HOLYSONAR")
	
	self.db = AceDB:New("HNSDB", HolySonar.DefaultOptions, true)
	self:setupOptions()

	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	self.Radar = newRadar()
	self.Radar:ApplyOptions()
	self.HolyNovaIcon = newHNIcon()
	self.HolyNovaIcon:ApplyOptions()
	self.frame:SetScript("OnUpdate", self.Radar.onUpdate)
end