if not HolySonar then return end
local HolySonar = HolySonar
local AceConfig = LibStub("AceConfig-3.0") -- For the options panel
local AceConfigDialog = LibStub("AceConfigDialog-3.0") -- Also for options panel
local AceDB = LibStub("AceDB-3.0") -- Makes saving things really easy
local AceDBOptions = LibStub("AceDBOptions-3.0") -- More database options

function HolySonar:setupOptions()
	HolySonar.options.args.profile = AceDBOptions:GetOptionsTable(HolySonar.db)
	AceConfig:RegisterOptionsTable("HolyNovaSonar", HolySonar.options, nil)

	HolySonar.optionsFrames = {}
	HolySonar.optionsFrames.general = AceConfigDialog:AddToBlizOptions("HolyNovaSonar", "Holy Nova Sonar")
end

function HolySonar:optionsChanged() -- notify the gui that the options changed
    LibStub("AceConfigRegistry-3.0"):NotifyChange("HolyNovaSonar")
end

local function setcolor(option, r, g, b, a)
	option.r = r
	option.g = g
	option.b = b
	if a then
		option.a = a
	end
	HolySonar.HolyNovaIcon:ApplyOptions()
	HolySonar.Radar:ApplyOptions() 
end

local anchors = {       
	CENTER = "CENTER",
	TOP = "TOP",
	BOTTOM = "BOTTOM",
	LEFT = "LEFT",
	RIGHT = "RIGHT",
	TOPLEFT = "TOPLEFT",
	TOPRIGHT = "TOPRIGHT",
	BOTTOMLEFT = "BOTTOMLEFT",
	BOTTOMRIGHT = "BOTTOMRIGHT",
}

local Order = 0
local function getOrder()
	Order = Order +1
	return Order
end


local function setcolor(option, r, g, b, a)
	option.r = r
	option.g = g
	option.b = b
	if a then
		option.a = a		
	end		
	HolySonar.HolyNovaIcon:ApplyOptions()
	HolySonar.Radar:ApplyOptions() 
end

local function setCircleTexture(texture)
	HolySonar.db.profile.radar.circleTexture = texture
	HolySonar.Radar:ApplyOptions() 
end

local function setLinesTexture(texture)
	HolySonar.db.profile.radar.linesTexture = texture
	HolySonar.Radar:ApplyOptions() 
end


	
HolySonar.DefaultOptions = {
	profile = {
		HolyNovaOverhealCoheficient = 0.9, -- moothz
		lock = false,
		refreshRate = 0.1, --call range checking function ever RefreshRate in seconds
		icon = {
			scale = 1,
			size = 64,
			offsetX = 400,
			offsetY = 134,
			anchor = "CENTER",
			fontSize = 18,
			enable = true,
			color = {
				BAD = { r = 0.674509804, g = 0.145098039, b = 0.043137255, a = 0.2},
				GOOD = { r = 0.309803922, g = 0.674509804, b = 0.043137255, a = 0.2}
			}
		},
		radar = {
			circleTexture = "Interface\\AddOns\\HolyNovaSonar\\textures\\diamond.blp",
			linesTexture = "Interface\\AddOns\\HolyNovaSonar\\textures\\lines1.blp",
			size = 200,
			scale = 1,
			offsetX = 400,
			offsetY = 0,
			dotScale = 1,
			color = {
				green = { r = 0.309803922, g = 0.674509804, b = 0.043137255}, 
				yellow = { r = 0.674509804, g = 0.592156863, b = 0.043137255},
				orange = { r = 0.674509804, g = 0.325490196, b = 0.043137255},
				red = { r = 0.674509804, g = 0.145098039, b = 0.043137255},
				radar = { r = 0, g = 1.0, b = 0, a = 1},
				background = { r = 0, g = 0, b = 0, a = 0},
				lines = { r = 1, g = 1, b = 1, a = 0.3}
			},
			enable = true,
			anchor = "CENTER",
			showOutsideRadar = true,       -- Show Units outside moothz
			outsideRange = 10,             -- How much to extend Holy nova range to show players outside of radar
			rotate = true, -- true = radar will rotate (like DBM one), false = radar will stay static (like the minimap)
			pitch = false,
		},
	},
}

HolySonar.options = {
	type = "group",
	childGroups = "tab",
	args = {
		headerGlobal = {
			name = "Global Settings",
			type = "header",
			order = getOrder()
		},
		lock = {
			name = "Lock",
			desc = "Locks the positions of the frames",
			type = "toggle",
			set = function(info,val)
				HolySonar.db.profile.lock = val
				HolySonar.db.profile.ClickThrough = val
				HolySonar.Radar:EnableMouse(not val)
				HolySonar.HolyNovaIcon:EnableMouse(not val)
			end,
			get = function(info) return HolySonar.db.profile.lock end,
			order = getOrder(),
			width = "full"
		},
		refreshRate = {
			name = "Refresh Rate",
			desc = "How often the radar and icon will be updated. less means more updates per second at a cost of performance.",
			type = "range",
			min = 0.05,
			max = 0.5,
			step = 0.05,
			set = function(info,val)
				HolySonar.db.profile.refreshRate = val                        
			end,
			get = function(info) return HolySonar.db.profile.refreshRate end,                
			order = getOrder(),
			width = "full"
		},
		
		------------- icon --------------   
		
		icon = {
			order = getOrder(),
			name = "Icon Settings",
			type = "group",
			args = {                   
				enable = {
					name = "Enable",
					desc = "Show the icon",
					type = "toggle",
					set = function(info,val) 
						HolySonar.db.profile.icon.enable = val
						HolySonar.HolyNovaIcon:ApplyOptions()
					end,
					get = function(info) return HolySonar.db.profile.icon.enable end,
					order = getOrder(),
					width = "full"
				},          
				size = {
					name = "Size",
					type = "range",
					min = 10,
					max = 400,
					step = 1,
					set = function(info,val)
						HolySonar.db.profile.icon.size = val
						HolySonar.HolyNovaIcon:Resize(val)                        
					end,
					get = function(info) return HolySonar.db.profile.icon.size end,
					order = getOrder()
				},      
				scale = {
					name = "Scale",
					type = "range",
					min = 1,
					max = 4,
					step = 0.1,
					set = function(info,val)
						HolySonar.db.profile.icon.scale = val
						HolySonar.HolyNovaIcon:SetScale(val)                          
					end,
					get = function(info) return HolySonar.db.profile.icon.scale end,
					order = getOrder(),
				},
				ph =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},
				--[[anchor = {              
					type = "select",
					name = "anchor",
					values = anchors,
					get = function() return HolySonar.db.profile.icon.anchor end,
					set = function(info, val)
						HolySonar.db.profile.icon.anchor = val
					end,
					order = getOrder(),
				},
				anchorBreak =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},		
				offsetX = {
					name = "Position X",
					type = "range",
					min = -4000,
					max = 4000,
					step = 1,
					set = function(info,val) 
						HolySonar.db.profile.icon.offsetX = val
						HolySonar.HolyNovaIcon:ClearAllPoints()
						HolySonar.HolyNovaIcon:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.icon.offsetX, HolySonar.db.profile.icon.offsetY);
					end,
					get = function(info) return math.floor(HolySonar.db.profile.icon.offsetX) end,
					order = getOrder()
				},		
				posxBreak =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},		
				offsetY = {
					name = "Position Y",
					type = "range",
					min = -4000,
					max = 4000,
					step = 1,
					set = function(info,val)
						HolySonar.db.profile.icon.offsetY = val
						HolySonar.HolyNovaIcon:ClearAllPoints()
						HolySonar.HolyNovaIcon:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.icon.offsetX, HolySonar.db.profile.icon.offsetY);
					end,
					get = function(info) return math.floor(HolySonar.db.profile.icon.offsetY) end,
					order = getOrder()
				},]]
				posyBreak =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},
				fontSize = {
					name = "Font Size",
					type = "range",
					min = 10,
					max = 25,
					step = 1,
					set = function(info,val)
						HolySonar.db.profile.icon.fontSize = val
						HolySonar.HolyNovaIcon:ApplyOptions()                     
					end,
					get = function(info) return math.floor(HolySonar.db.profile.icon.fontSize) end,   
					order = getOrder()
				},
				ph2 =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},
				colorBAD = {
					name = "Not Optimal",
					desc = "The icon will have an overlay with this color whenever it is not optimal to use Holy Nova",
					type = "color",
					get = function()
					return HolySonar.db.profile.icon.color.BAD.r, HolySonar.db.profile.icon.color.BAD.g, HolySonar.db.profile.icon.color.BAD.b, HolySonar.db.profile.icon.color.BAD.a end,
					set = function(info, r, g, b, a) setcolor(HolySonar.db.profile.icon.color.BAD, r, g, b, a) end,
					hasAlpha = true,
					order = getOrder()
				},
				colorGOOD = {
					name = "Optimal Usage",
					desc = "The icon will have an overlay with this color whenever it is optimal to use Holy Nova",
					type = "color",
					get = function() return HolySonar.db.profile.icon.color.GOOD.r, HolySonar.db.profile.icon.color.GOOD.g, HolySonar.db.profile.icon.color.GOOD.b, HolySonar.db.profile.icon.color.GOOD.a end,
					set = function(info, r, g, b, a) setcolor(HolySonar.db.profile.icon.color.GOOD, r, g, b, a) end,
					hasAlpha = true,
					order = getOrder()
				},
			}
		},
		
		------------- radar --------------
		
		radar = {
			order = getOrder(),
			name = "Radar Settings",
			type = "group",
			args = {           
				enable = {
					name = "Enable",
					desc = "Show the radar",
					type = "toggle",
					set = function(info,val) 
						HolySonar.db.profile.radar.enable = val; 
						HolySonar.Radar:ApplyOptions() 
						end,
					get = function(info) return HolySonar.db.profile.radar.enable end,
					order = getOrder(),
					width = "full"
				},  
				rotate = {
					name = "Rotate in player direction",
					desc = "Rotate radar in player direction",
					type = "toggle",
					set = function(info,val) HolySonar.db.profile.radar.rotate = val end,
					get = function(info) return HolySonar.db.profile.radar.rotate end,
					order = getOrder(),
					width = "full"
				},  
				showOutsideRadar = {
					name = "Show players outside of Holy Nova range", -- TODO implement
					desc = "Show players furhter than the holy nova range. Displays them outside of the radar circle.",
					type = "toggle",
					set = function(info,val) HolySonar.db.profile.radar.showOutsideRadar = val end,
					get = function(info) return HolySonar.db.profile.radar.showOutsideRadar end,
					order = getOrder(),
					width = "full"
				}, 
				pitch = {
					name = "Pitch the Radar", -- TODO implement
					desc = "Skews the radar vertically to simulate a 3D map",
					type = "toggle",
					set = function(info,val) HolySonar.db.profile.radar.pitch = val end,
					get = function(info) return HolySonar.db.profile.radar.pitch end,
					order = getOrder(),
					width = "full"
				},
				outsideRange = {
					name = "Max Range",
					desc = "Yards to show players outside of the radar",
					type = "range",
					min = 0,
					max = 30,
					step = .5,
					set = function(info,val)
						HolySonar.db.profile.radar.outsideRange = val                        
					end,
					get = function(info) return HolySonar.db.profile.radar.outsideRange end,                
					order = getOrder()
				},
				ph3 =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},  
				size = {
					name = "Size",
					type = "range",
					min = 50,
					max = 800,
					step = 1,
					set = function(info,val)
						HolySonar.db.profile.radar.size = val
						HolySonar.Radar:Resize(val)                       
					end,
					get = function(info) return HolySonar.db.profile.radar.size end,
					order = getOrder()
				},      
				scale = {
					name = "Scale",
					type = "range",
					min = 1,
					max = 4,
					step = 0.1,
					set = function(info,val)
						HolySonar.db.profile.radar.scale = val
						HolySonar.Radar:SetScale(val)                         
					end,
					get = function(info) return HolySonar.db.profile.radar.scale  end,
					order = getOrder(),
				},
				--[[ph4 =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},
				anchor = {             
					type = "select",
					name = "anchor",
					values = anchors,
					get = function() return HolySonar.db.profile.radar.anchor end,
					set = function(info, val)
						HolySonar.db.profile.radar.anchor = val
					end,
					order = getOrder(),
				},              
				offsetX = {
					name = "Position X",
					type = "range",
					min = -4000,
					max = 4000,
					step = 1,
					set = function(info,val) 
						HolySonar.db.profile.radar.rPosx = val      
						HolySonar.Radar:ClearAllPoints()
						HolySonar.Radar:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.radar.offsetX, HolySonar.db.profile.radar.offsetY);
					end,
					get = function(info) return math.floor(HolySonar.db.profile.radar.offsetX) end,
					order = getOrder()
				},
				offsetY = {
					name = "Position Y",
					type = "range",
					min = -4000,
					max = 4000,
					step = 1,
					set = function(info,val)
						HolySonar.db.profile.radar.offsetY = val
						HolySonar.Radar:ClearAllPoints()
						HolySonar.Radar:SetPoint("CENTER", UIParent, "CENTER", HolySonar.db.profile.radar.offsetX, HolySonar.db.profile.radar.offsetY);
					end,
					get = function(info) return math.floor(HolySonar.db.profile.radar.offsetY) end,
					order = getOrder()
				},]]
				rPosyBreak =  {
					type = "description",
					name = "",
					order = getOrder(),
					width="full"
				},	
				ph5 =  {
					type = "description",
					name = "Color for player health deficit (dots)",
					order = getOrder(),
					width="full"
				},          
				colorgreen = {
					name = "Full HP",
					type = "color",
					get = function() return HolySonar.db.profile.radar.color.green.r, HolySonar.db.profile.radar.color.green.g, HolySonar.db.profile.radar.color.green.b end,
					set = function(info, r, g, b) setcolor(HolySonar.db.profile.radar.color.green, r, g, b) end,
					order = getOrder()
				},
				coloryellow = {
					name = "Low Damage",
					type = "color",
					get = function() return HolySonar.db.profile.radar.color.yellow.r, HolySonar.db.profile.radar.color.yellow.g, HolySonar.db.profile.radar.color.yellow.b end,
					set = function(info, r, g, b) setcolor(HolySonar.db.profile.radar.color.yellow, r, g, b) end,
					order = getOrder()
				},          
				colororange = {
					name = " Moderate Damage",
					type = "color",
					get = function() return HolySonar.db.profile.radar.color.orange.r, HolySonar.db.profile.radar.color.orange.g, HolySonar.db.profile.radar.color.orange.b end,
					set = function(info, r, g, b) setcolor(HolySonar.db.profile.radar.color.orange, r, g, b) end,
					order = getOrder()
				},
				colorred = {
					name = "High Damage",
					type = "color",
					get = function() return HolySonar.db.profile.radar.color.red.r, HolySonar.db.profile.radar.color.red.g, HolySonar.db.profile.radar.color.red.b end,
					set = function(info, r, g, b) setcolor(HolySonar.db.profile.radar.color.red, r, g, b) end,
					order = getOrder()
				},
				ph6 =  {
					type = "description",
					name = "Color for Radar Appearance",
					order = getOrder(),
					width="full"
				},  
				colorradar = {
					name = "Radar",
					type = "color",
					get = function() return HolySonar.db.profile.radar.color.radar.r, HolySonar.db.profile.radar.color.radar.g, HolySonar.db.profile.radar.color.radar.b, HolySonar.db.profile.radar.color.radar.a end,
					set = function(info, r, g, b, a) setcolor(HolySonar.db.profile.radar.color.radar, r, g, b, a); end,
					order = getOrder(),
					hasAlpha = true
				},
				colorBackground = {
					name = "Background",
					type = "color",
					get = function() return HolySonar.db.profile.radar.color.background.r, HolySonar.db.profile.radar.color.background.g, HolySonar.db.profile.radar.color.background.b, HolySonar.db.profile.radar.color.background.a end,
					set = function(info, r, g, b, a) setcolor(HolySonar.db.profile.radar.color.background, r, g, b, a); end,
					order = getOrder(),
					hasAlpha = true
				},
				colorLines = {
					name = "Lines",
					type = "color",
					get = function() return HolySonar.db.profile.radar.color.lines.r, HolySonar.db.profile.radar.color.lines.g, HolySonar.db.profile.radar.color.lines.b, HolySonar.db.profile.radar.color.lines.a end,
					set = function(info, r, g, b, a) setcolor(HolySonar.db.profile.radar.color.lines, r, g, b, a); end,
					order = getOrder(),
					hasAlpha = true
				},
				radarTexture = {
					name = "Radar Texture",
					desc = "Radar Circle Texture",
					type = "select",
					values = {
						["Interface\\AddOns\\HolyNovaSonar\\textures\\radial.blp"] = "Radial", 
						["Interface\\AddOns\\HolyNovaSonar\\textures\\diamond.blp"] = "Diamond", 
						["Interface\\AddOns\\HolyNovaSonar\\textures\\angle.blp"] = "Angle" },
					get = function()  return HolySonar.db.profile.radar.circleTexture end,
					set = function(_,texture) setCircleTexture(texture) end,
					order = getOrder()
				},
				linesTex = {
					name = "Lines Texture",
					desc = "Lines overlay texture",
					type = "select",
					values = {
						["Interface\\AddOns\\HolyNovaSonar\\textures\\lines1.blp"] = "Circles with Cross",
						["Interface\\AddOns\\HolyNovaSonar\\textures\\lines2.blp"] = "Circles Only",
						["Interface\\AddOns\\HolyNovaSonar\\textures\\lines3.blp"] = "Cross Only",
						["Interface\\AddOns\\HolyNovaSonar\\textures\\blank.blp"] = "No Lines"},
					get = function()  return HolySonar.db.profile.radar.linesTexture end,
					set = function(_,texture) setLinesTexture(texture) end,
					order = getOrder()
				}
			}
		}
	}
}

